#pragma once

#include <ranges>
#include <algorithm> // For std::transform
#include <utility>   // For std::pair
#include <Eigen/Dense>
#include <functional>
// Helper function to create an enumerator
// auto enumerate(const auto& container) {
//     auto indices = std::views::iota(0, static_cast<int>(container.size()));
//     return std::views::zip(indices, container);
// }
void bresenham(const Eigen::Vector2i& start, const Eigen::Vector2i& end, std::function<void(const Eigen::Vector2i&)> f) {
    std::vector<Eigen::Vector2i> linePoints;

    int x0 = start.x(), y0 = start.y();
    int x1 = end.x(), y1 = end.y();

    int dx = std::abs(x1 - x0);
    int dy = -std::abs(y1 - y0);

    int sx = (x0 < x1) ? 1 : -1;
    int sy = (y0 < y1) ? 1 : -1;
    int err = dx + dy;

    while (true) {
        f(Eigen::Vector2i(x0, y0));
        if (x0 == x1 && y0 == y1) break;

        int e2 = 2 * err;
        if (e2 >= dy) {
            err += dy;
            x0 += sx;
        }
        if (e2 <= dx) {
            err += dx;
            y0 += sy;
        }
    }

}
